/**
 * 
 */
package uk.org.wuglug.gryle.ui.m;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import javax.swing.JTable;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableModel;

/**
 * @author Richard Warburton
 * A Singleton style model for a table of songs
 */
public class SongTableModel extends AbstractTableModel implements Serializable {

	private static final String _FILE_LOCATION = System.getProperty("user.home")+File.separator+".kavanagh";
	
	// SHUTDOWN THREAD ------------------------
	
	class ModelSaverThread extends Thread implements Serializable {
		
		/**
		 * @see java.lang.Thread#run()
		 */
		@Override
		public void run() {
			this.songTableModel.save();
		}

		private SongTableModel songTableModel;
		
		ModelSaverThread(SongTableModel songTableModel) {
			this.songTableModel = songTableModel;
		}
	}
	
	
	// STATIC DECLARATIONS ------------------------
	
	private final static int _COLUMN_COUNT = 5;
	private static SongTableModel inst = load();
	
	/**
	 * @return the inst
	 */
	public static SongTableModel getInst() {
		return inst;
	}
	
	// FILE OPERATIONS ------------------------
	
	/**
	 * Serialises data into a file
	 */
	public void save() {
		try {
			ObjectOutputStream out = new ObjectOutputStream(new FileOutputStream(new File(_FILE_LOCATION)));
			out.writeObject(this);
			out.flush();
			out.close();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * Called initially to load up data store from file
	 */
	private static SongTableModel load() {
		File file = new File(_FILE_LOCATION);
		if(file.exists() && file.canRead()) {
			System.out.println("Loading Model from File");
			try {
				ObjectInputStream in = new ObjectInputStream(new FileInputStream(_FILE_LOCATION));
				SongTableModel stm = (SongTableModel)in.readObject();
				in.close();
				return stm;
			} catch (FileNotFoundException e) {
				e.printStackTrace();
			} catch (IOException e) {
				e.printStackTrace();
			} catch (ClassNotFoundException e) {
				e.printStackTrace();
			}
			return null;
		} else {
			System.out.println("No Model to load from");
			return new SongTableModel();
		}
	}

	//  INSTANCE METHODS ------------------------
	
	private List<SongModel> data;
	private List<SongModel> model;
	private JTable view;
	
	private SongTableModel() {
		this.data = new ArrayList<SongModel>();
		this.model = new ArrayList<SongModel>();
		Collections.copy(this.data, this.model);
		
		Runtime.getRuntime().addShutdownHook(new ModelSaverThread(this));
	}
	
	public void addAll(List<File> contents) {
		
		for(File f:contents) {
			SongModel song = new SongModel(f);
			this.data.add(song);
			this.model.add(song);
		}
		
		fireTableDataChanged();
		
	}
	
	public int getColumnCount() {
		return _COLUMN_COUNT;
	}

	public int getRowCount() {
		return this.model.size();
	}

	public Object getValueAt(int row, int col) {
		return this.model.get(row).getVal(col);
	}
	
	public void sort(int column) {
		Comparator comparator = SongModel.comparators[column];
		Collections.sort(this.model, comparator);
		fireTableDataChanged();
	}
	
	/**
	 * Updates the data model to only display those songs that match
	 * this string pattern.
	 * @param pattern
	 */
	public void filter(String pattern) {
		System.out.println(pattern);
		this.model.clear();
		for (SongModel model : this.data) {
			if(model.match(pattern)) {
				this.model.add(model);
			}
		}
		fireTableDataChanged();
	}

	/**
	 * Assumes view has been set
	 *
	 */
	public boolean delete() {
		int row = this.view.getSelectedRow();
		SongModel val = this.model.remove(row);
		this.data.remove(val);
		boolean ret = val.delete();
		fireTableDataChanged();
		return ret;
	}

	/**
	 * Assumes view has been set
	 *
	 */
	public void remove() {
		int row = this.view.getSelectedRow();
		SongModel val = this.model.remove(row);
		this.data.remove(val);
		fireTableDataChanged();
	}

	/**
	 * @param view the view to set
	 */
	public void setView(JTable view) {
		this.view = view;
	}

	public SongModel getSong(int row) {
		return this.model.get(row);
	}
}
